import unittest
from mock import create_autospec, call
import backup


def mock_action_get(key):
    return {"workingDir": "this/dir", "args": "-arg1 -arg2"}[key]


class TestBackups(unittest.TestCase):

    def test_dump(self):
        mkdir = create_autospec(backup.mkdir, return_value=True)
        execute = create_autospec(backup.execute, return_value="output")
        action_set = create_autospec(backup.action_set, return_value=None)
        action_fail = create_autospec(backup.action_fail, return_value=None)
        backup.mkdir = mkdir
        backup.execute = execute
        backup.action_set = action_set
        backup.action_fail = action_fail
        backup.action_get = mock_action_get

        backup.dump()

        mkdir.assert_called_once_with("this/dir")
        execute.assert_called_once_with("mongodump -arg1 -arg2", "this/dir")
        action_set.assert_has_calls([
            call({"command": "mongodump -arg1 -arg2",
                 "working-dir": "this/dir"}),
            call({"output": "output"})])

    def test_restore(self):
        mkdir = create_autospec(backup.mkdir, return_value=True)
        execute = create_autospec(backup.execute, return_value="output")
        action_set = create_autospec(backup.action_set, return_value=None)
        action_fail = create_autospec(backup.action_fail, return_value=None)
        backup.mkdir = mkdir
        backup.execute = execute
        backup.action_set = action_set
        backup.action_fail = action_fail

        backup.restore()

        mkdir.assert_called_once_with("this/dir")
        execute.assert_called_once_with("mongorestore -arg1 -arg2", "this/dir")
        action_set.assert_has_calls([
            call({"command": "mongodump -arg1 -arg2",
                 "working-dir": "this/dir"}),
            call({"output": "output"})])

if __name__ == '__main__':
    unittest.main()
