#!/usr/bin/env python3

import amulet
from pymongo import MongoClient
from collections import Counter


#########################################################
# Test Quick Config
#########################################################
scale = 2
seconds = 1800

# amount of time to wait before testing for replicaset
# status
wait_for_replicaset = 60*5
# amount of time to wait for the data relation
wait_for_relation = 60*5

#########################################################
# 3shard cluster configuration
#########################################################
d = amulet.Deployment(series='trusty')

d.add('mongodb', units=scale, series='trusty',
      constraints={'root-disk': '20480M'})
d.add('storage', charm='cs:~chris-gondolin/trusty/storage-5', series='trusty')
d.configure('storage', {'provider': 'local'})

d.expose('mongodb')

# Perform the setup for the deployment.
try:
    d.setup(seconds)
    d.sentry.wait(wait_for_replicaset)
except amulet.helpers.TimeoutError:
    message = 'The environment did not setup in %d seconds.', seconds
    amulet.raise_status(amulet.SKIP, msg=message)
except:
    raise

ordered_units = sorted(d.sentry['mongodb'], key=lambda u: u.info['unit'])
sentry_dict = {
    'mongodb0-sentry': ordered_units[0],
    'mongodb1-sentry': ordered_units[1]
}


#############################################################
# Check agent status
#############################################################
def validate_status():
    d.sentry.wait_for_status(d.juju_env, ['mongodb'])


#############################################################
# Validate proper replicaset setup
#############################################################
def validate_replicaset_setup():

    d.sentry.wait(seconds)

    unit_status = []

    for service in sentry_dict:
        addy = sentry_dict[service].info['public-address']
        if ":" in addy:
            addy = "[{}]".format(addy)
        client = MongoClient(addy)
        r = client.admin.command('replSetGetStatus')
        unit_status.append(r['myState'])
        client.close()

    primaries = Counter(unit_status)[1]
    if primaries != 1:
        message = "Only one PRIMARY unit allowed! Found: %s" % (primaries)
        amulet.raise_status(amulet.FAIL, message)

    secondrs = Counter(unit_status)[2]
    if secondrs != 1:
        message = "Only one SECONDARY unit allowed! (Found %s)" % (secondrs)
        amulet.raise_status(amulet.FAIL, message)


validate_status()
validate_replicaset_setup()
print("Adding storage relation, and sleeping for 2 min.")
try:
    d.relate('mongodb:data', 'storage:data')
except OSError as e:
    print("ignoring error:{}", e)
d.sentry.wait(wait_for_relation)
validate_status()
validate_replicaset_setup()
